import 'dart:convert';
import 'package:flutter/services.dart' show rootBundle;
import 'package:flutter_apns/apns.dart';
import 'package:flutter_apns/flutter_apns.dart';
import 'package:pam_flutter/models/pam_message_model.dart';
import 'package:pam_flutter/pam_flutter.dart';
import 'package:pam_flutter/types.dart';

class PamNotification {
  PamNotification._();

  static String? _appPrefix;
  static late PushConnector _connector;

  static Future<String> appPrefix() async {
    if (_appPrefix == null) {
      String raw = await rootBundle.loadString("pam-config.json");
      Map<String, dynamic> cfg = json.decode(raw);
      _appPrefix = cfg["app-prefix"];
    }
    return _appPrefix!;
  }

  static Future<void> pamInitial() async {
    _connector = createPushConnector();
    _connector.configure(
      onLaunch: _onLaunchMessage,
      onResume: _onResumeMessage,
      onMessage: _onMessageMessage,
    );
  }

  static Future<void> askNotificationPermission() async {
    _connector.requestNotificationPermissions();
    if (_connector.token.value != null) {
      Pam.trackSavePush(token: _connector.token.value!);
    }
  }

  static Future<void> _onLaunchMessage(RemoteMessage message) async {
    print("_onLaunchMessage = ${message.data}");
  }

  static Future<void> _onResumeMessage(RemoteMessage message) async {
    print("_onResumeMessage = ${message.data}");
  }

  static Future<void> _onMessageMessage(RemoteMessage message) async {
    print("_onMessageMessage = ${message.data}");
  }

  static Future<bool> isPamMessageReceived(
      Map<String, dynamic>? message) async {
    if (message == null) {
      return false;
    }
    if (message["pam"] == null) {
      return false;
    }

    Map<String, dynamic> rawPam = json.decode(message["pam"] ?? "{}");
    PamMessageModel pam = PamMessageModel.fromJson(await appPrefix(), rawPam);

    Pam.dispatch(PamStandardCallback.on_message, pam.toMap());
    return true;
  }
}
